# -*- coding: utf-8 -*-

from copy import copy
import json
from pyrocumulus.conf import settings
from pyrocumulus.web.applications import Application, URLSpec
from tornado import gen
from tornado.web import RequestHandler
from jaobi.models import ContentConsumption


class _SizedConsumptionCache:

    def __init__(self):
        self.cache = []
        self.cache_size = settings.CONSUMPTION_CACHE_SIZE
        self.lock = None

    @gen.coroutine
    def add(self, obj):
        self.cache.append(obj)
        if len(self.cache) >= self.cache_size and not self.lock:
            self.lock = 'lóqui'
            to_insert = copy(self.cache)
            self.cache = []
            yield self.insert(to_insert)
            from jaobi.utils import log
        self.lock = None

    @gen.coroutine
    def insert(self, obj_list):
        yield ContentConsumption.objects.insert(obj_list)


sized_cache = _SizedConsumptionCache()


def get_cache(cache_type='sized'):
    return sized_cache


# Here a simple app to show the size of the cache queue
# Mainly used for testing and debugging
class SpyHandler(RequestHandler):

    def get(self):
        mydict = {'size': len(sized_cache.cache),
                  'locked': bool(sized_cache.lock)}
        self.write(json.dumps(mydict))


spy = URLSpec('/cache/spy', SpyHandler)
spyapp = Application(url_prefix='', extra_urls=[spy])



# -*- coding: utf-8 -*-

from zmq.eventloop import ioloop

ioloop.install()

from pyrocumulus.db import MongoConnection

connection = MongoConnection()
connection.connect()


VERSION = '0.14.2.3.post1'



# -*- coding: utf-8 -*-

# The reports api
from tornado import gen
from jaobi.reports.reduced import (SiteConsumers, SiteConsumption,
                                   ThemeConsumers, ThemeConsumption,
                                   ContentConsumptionReport,
                                   ContentConsumersReport,
                                   ReferrerConsumption)


@gen.coroutine
def site_consumers(**kwargs):
    """Returns consolidated report about consumers on sites.

    :param kwargs: Filter to queryset."""

    r = yield SiteConsumers.get_sites_report(**kwargs)
    return r


@gen.coroutine
def site_consumers_history(**kwargs):
    """Returns history report about consumers on sites.

    :param kwargs: Filter to queryset."""

    r = yield SiteConsumers.get_sites_report_history(**kwargs)
    return r


@gen.coroutine
def site_consumption(**kwargs):
    """Returns consolidated report about consumption on sites.

    :param kwargs: Filter to queryset."""

    r = yield SiteConsumption.get_sites_report(**kwargs)
    return r


@gen.coroutine
def site_consumption_history(**kwargs):
    """Returns history report about consumption on sites.

    :param kwargs: Filter to queryset."""

    r = yield SiteConsumption.get_sites_report_history(**kwargs)
    return r


@gen.coroutine
def theme_consumers(**kwargs):
    """Returns consolidated report about consumers on themes.

    :param kwargs: Filter to queryset."""

    r = yield ThemeConsumers.get_themes_report(**kwargs)
    return r


@gen.coroutine
def theme_consumers_history(**kwargs):
    """Returns history report about consumers on themes.

    :param kwargs: Filter to queryset."""

    r = yield ThemeConsumers.get_themes_report_history(**kwargs)
    return r


@gen.coroutine
def theme_consumption(**kwargs):
    """Returns consolidated report about consumers on themes.

    :param kwargs: Filter to queryset."""

    r = yield ThemeConsumption.get_themes_report(**kwargs)
    return r


@gen.coroutine
def theme_consumption_history(**kwargs):
    """Returns history report about consumers on themes.

    :param kwargs: Filter to queryset."""

    r = yield ThemeConsumption.get_themes_report_history(**kwargs)
    return r


@gen.coroutine
def content_consumers(**kwargs):
    """Returns consolidated report about consumers on content.

    :param kwargs: Filter to queryset."""

    r = yield ContentConsumersReport.get_content_report(**kwargs)
    return r


@gen.coroutine
def content_consumption(**kwargs):
    """Returns consolidated report about consumption on content.

    :param kwargs: Filter to queryset."""

    r = yield ContentConsumptionReport.get_content_report(**kwargs)
    return r


@gen.coroutine
def referrer_consumption(**kwargs):
    """Returns consolidated report about consumption originated from referrer.

    :param kwargs: Filter to queryset."""

    r = yield ReferrerConsumption.get_referrer_report(**kwargs)
    return r



# -*- coding: utf-8 -*-

# This module contains the 'live' reports about Jaobi usage.

import datetime
from tornado import gen
from jaobi.models import ContentConsumption


def _format_result(result):
    results = result['result']
    rdict = {}
    for r in results:
        rdict[r['_id']] = r['total']
    return rdict


class RealTimeConsumption:

    """ A class responsible for real time reports about consumption.

    Here 'real time' means all the ContentConsumption that was
    included in the last 5 minutes and don't have an unload_date."""

    base_collection = ContentConsumption

    @classmethod
    @gen.coroutine
    def get_themes_report(cls, **kwargs):
        """ Returns report for realtime theme consumption.

        :param kwargs: Arguments to filter the queryset before
          aggregating it. Note that ``inclusion_date__gte`` will
          always be five minutes ago and ``unload_date`` will always
          be None."""

        now = datetime.datetime.now()
        five_min_ago = now - datetime.timedelta(minutes=5)
        kwargs['inclusion_date__gte'] = five_min_ago
        kwargs['unload_date'] = None

        unwind = {'$unwind': '$themes'}
        group = {'$group':
                 {'_id': '$themes',
                  'total': {'$sum': 1}}}
        sort = {'$sort': {'total': -1}}

        pipeline = [unwind, group, sort]
        qs = cls.base_collection.objects.filter(**kwargs)
        r = yield qs.aggregate(*pipeline)
        return _format_result(r)
